@@ -82,8 +82,17 @@ static void dump_feature(const struct feature_results *results,
 		 feature_on(results->wanted, idx))
 		suffix = feature_on(results->wanted, idx) ?
 			" [requested on]" : " [requested off]";
-	printf("%s%s: %s%s\n", prefix, name,
-	       feature_on(results->active, idx) ? "on" : "off", suffix);
+	if (is_json_context()) {
+		open_json_object(name);
+		print_bool(PRINT_JSON, "active", NULL, feature_on(results->active, idx));
+		print_bool(PRINT_JSON, "fixed", NULL,
+			   (!feature_on(results->hw, idx) || feature_on(results->nochange, idx)));
+		print_bool(PRINT_JSON, "requested", NULL, feature_on(results->wanted, idx));
+		close_json_object();
+	} else {
+		printf("%s%s: %s%s\n", prefix, name,
+		       feature_on(results->active, idx) ? "on" : "off", suffix);
+	}
 }
 
 /* this assumes pattern contains no more than one asterisk */
@@ -153,9 +162,18 @@ int dump_features(const struct nlattr *const *tb,
 					feature_on(results.active, j);
 			}
 		}
-		if (n_match != 1)
-			printf("%s: %s\n", off_flag_def[i].long_name,
-			       flag_value ? "on" : "off");
+		if (n_match != 1) {
+			if (is_json_context()) {
+				open_json_object(off_flag_def[i].long_name);
+				print_bool(PRINT_JSON, "active", NULL, flag_value);
+				print_null(PRINT_JSON, "fixed", NULL, NULL);
+				print_null(PRINT_JSON, "requested", NULL, NULL);
+				close_json_object();
+			} else {
+				printf("%s: %s\n", off_flag_def[i].long_name,
+				       flag_value ? "on" : "off");
+			}
+		}
 		if (n_match == 0)
 			continue;
 		for (j = 0; j < results.count; j++) {
@@ -210,8 +228,10 @@ int features_reply_cb(const struct nlmsghdr *nlhdr, void *data)
 
 	if (silent)
 		putchar('\n');
-	printf("Features for %s:\n", nlctx->devname);
+	open_json_object(NULL);
+	print_string(PRINT_ANY, "ifname", "Features for %s:\n", nlctx->devname);
 	ret = dump_features(tb, feature_names);
+	close_json_object();
 	return (silent || !ret) ? MNL_CB_OK : MNL_CB_ERROR;
 }
 
@@ -234,7 +254,12 @@ int nl_gfeatures(struct cmd_context *ctx)
 				      ETHTOOL_FLAG_COMPACT_BITSETS);
 	if (ret < 0)
 		return ret;
-	return nlsock_send_get_request(nlsk, features_reply_cb);
+
+	new_json_obj(ctx->json);
+	ret = nlsock_send_get_request(nlsk, features_reply_cb);
+	delete_json_obj();
+
+	return ret;
 }
 
 /* FEATURES_SET */
